# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuição-NãoComercial-Compartilhada Igual 3.0 Não Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

import ConfigParser
import sys
import tty
import termios
import unicodedata


class TelnetEmulation(object):
    """
    Modulo principal. Formacao da tela, posicoes, cores, etc.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    def __init__(self):
        """
        Parametros iniciais.
        """

        # Atributos (os mais utilizados)
        self.normal = 0  # ´Reset´. Todos os atributos desligados
        self.bold = 1  # Negrito
        self.thin = 2  # Fraco. Intensidade reduzida (nao e amplamente suportado)
        self.italic = 3  # Italico (não e amplamente suportado)
        self.underline = 4  # Sublinhado
        self.blink = 5  # Pisca lento (nao e amplamente suportado)
        self.fastblink = 6  # Pisca rapido (nao e amplamente suportado)
        self.reverse = 7  # Inverte as cores
        self.conceal = 8  # Cor de fundo igual a cor do caracter
        self.strikeout = 9  # Cruzado. Carcateres legiveis mas 'cortado' (nao e amplamente suportado)

        # Cores de caracter (foreground)
        self.fg_black = 30  # Preto
        self.fg_red = 31  # Vermelho
        self.fg_green = 32  # Verde
        self.fg_yellow = 33  # Amarelo
        self.fg_blue = 34  # Azul
        self.fg_magenta = 35  # ´Magenta´ (rosa)
        self.fg_cyan = 36  # ´Cyan´ (azul claro)
        self.fg_white = 37  # Branco

        # Cores de fundo (background)
        self.bg_black = 40  # Preto
        self.bg_red = 41  # Vermelho
        self.bg_green = 42  # Verde
        self.bg_yellow = 43  # Amarelo
        self.bg_blue = 44  # Azul
        self.bg_magenta = 45  # ´Magenta´ (rosa)
        self.bg_cyan = 46  # ´Cyan´ (azul claro)
        self.bg_white = 47  # Branco

        # Alinhamento
        self.align_left = 0  # Esquerda
        self.align_right = 1  # Direita
        self.align_center = 2  # Centro

        self.beepsound = '\007'  # ´beep´ (sinal sonoro)

        # Níveis ( em ´exit()´)
        self.level = {'ok': 0, 'alert': 1, 'critical': 2, 'error': 3,
                      'warning': 4, 'notice': 5, 'info': 6, 'debug': 7}

        # Leitura do arquivo de configuracao
        cfg = ConfigParser.ConfigParser()
        cfg.read('lib/terminal.conf')

        # Parametros
        self.max_col = cfg.getint('main', 'max_col') if cfg.has_option('main', 'max_col') else 10
        self.max_col = 16 if self.max_col < 16 else self.max_col  # Valor minimo de colunas
        self.max_row = cfg.getint('main', 'max_row') if cfg.has_option('main', 'max_row') else 20
        self.max_row = 4 if self.max_row < 4 else self.max_row  # Valor minimo de linhas
        self.use_colors = cfg.getboolean('main', 'colors') if cfg.has_option('main', 'colors') else False
        self.col_truncate = cfg.getboolean('main', 'col_truncate')  if cfg.has_option('main', 'col_truncate') else False
        self.autoconfirm = 0 if not cfg.has_option('main', 'autoconfirm') else cfg.getboolean('main', 'autoconfirm')

        # Cor padrao do terminal
        if self.use_colors:
            self.colors = cfg.get('colors', 'color') if cfg.has_option('colors', 'color') else '37;40'
        else:
            self.colors = '37;40'

        # Remove todos os acentos
        if cfg.has_option('text', 'normalize'):
            self.normalize = cfg.getboolean('text', 'normalize')
        else:
            self.normalize = True


    def log_terminal(self, level='INFO', description=None, abend=False):
        """
        ´Log´.

        :param level: codigo de retorno,
        :param description: descricao do ´log´ ou erro,
        :param abend: {boolean} verdadeiro finaliza aplicacao,
        :return: nivel.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        if description and __name__ == '__main__':
            print 'Terminal %s: %s' % (level.upper(), description)

        if abend:
            exit(abend)

        return self.level.get(level)

    def say(self, row, col, text='', *colors):
        """
        Posiciona na linha e coluna e imprime o texto com os atributos de cores.

        :param row: numero da linha,
        :param col: numero da coluna,
        :param text: texto a ser impresso (´print´),
        :param colors: parametros de definição de cores,
        :return: nova posicao da coluna
        :see: atributos, cores do caracter, cores do fundo.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        # Mostra a linha
        line = '\033[{0:d};{1:d}f\033[{2:>}m{3:>}\033[0m'

        # Remove todos os acentos (padrao e normalize = 1)
        if self.normalize:
            try:
                text = unicodedata.normalize('NFKD', text.decode())  # utf-8
                text = text.encode('ASCII', 'ignore')
            except:
                pass

        text = text[0:self.max_col] if self.col_truncate else text

        if not self.use_colors:  # Remove atributos de cor
            colors = filter(lambda f: f < 30, colors)

        print line.format(row, col, ';'.join(map(lambda c: str(c), colors)), text)

        # Posicao atualizada da coluna
        col += len(text)

        return col

    def get(self, lin=1, col=1, mask=1, echo=1, go=0, pwd=False, stand=None):
        """
        Recebe uma sequencia de caracteres digitados e os concatena.

        :param lin: linha a ser utilizada para o ´print´,
        :param col: coluna a ser uitlizada para o ´print´,
        :param mask: quantidade de caracteres utilizado na mascara,
        :param echo: {boolean} verdadeiro (padrao) se os caracteres deverao ser mostrados no
        ´´print´´,
        :param go: {boolean} verdadeiro se ao final do ´print´ uma sequencia de nova linha devera
        ser executada (crlf),
        :param pwd: {boolean} verdadeiro para nao mostrar os caracteres digitados,
        :return: sequência de caracteres.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        __BKSPACE = 8
        __CRLF = 13  # ´Enter´
        __ESCAPE = 27  # ´Escape´
        __LEFT = 91
        __DEL = 127

        if not mask:
            go = True

        fd = sys.stdin.fileno()
        old_settings = termios.tcgetattr(fd)
        tty.setraw(fd)

        ret = ''
        ncol = col

        lineget = '\033[{0:d};{1:d}f\033[1;4m{2:>}\033[0m'

        if not stand:
            print lineget.format(lin, col, ' ' * mask)
        else:
            stand = str(stand).ljust(mask)
            print lineget.format(lin, col, stand)

        print '\033[{0:d};{1:d}f'.format(lin - 1, col)
        ft = True

        while True:
            ch = sys.stdin.read(1)

            # Primeira vez. Ao ser digitado o primeiro caracter
            if ft and ord(ch) != __CRLF:
                # Limpa a ultima linha, considerada como linha de mensagem
                self.erase_line(self.max_row)
                print lineget.format(lin, ncol, ' ' * mask)  # Mostra o campo sem digitacao
                ft = False

            if ord(ch) == __CRLF:
                if not ret:
                    ret = stand

                break
            elif ord(ch) == __ESCAPE:
                ret = chr(27)
                break
            elif ord(ch) in (__BKSPACE, __DEL):
                if ncol <= col:
                    ncol = col
                    ret = ''
                else:
                    if ncol >= col:
                        print lineget.format(lin, ncol - 1, ' ')
                    else:
                        print lineget.format(lin, ncol, ' ')
                    ncol -= 1
                    ret = ret[:-1]

                    if go and ncol == 0:
                        break
            else:
                if ncol > col + mask - 1:
                    ncol = col + mask - 1
                    ret = ret[:ncol] + ch
                else:
                    ret += ch

                if echo:
                    print lineget.format(lin, ncol, ch)
                else:
                    if pwd:
                        print lineget.format(lin, ncol, '*')  # Troca o caracter digitado por um '*'
                    else:
                        pass  # Nao mostra o caracter (normalmente para entrada de senha)

                if ncol >= (col + mask - 1) and go:
                    break

                ncol += 1

            dif = 1 if ncol > (col + mask - 1) else 0
            print '\033[{0:d};{1:d}f'.format(lin - 1, ncol - dif)  # Ajusta a posicao do cursor

        termios.tcsetattr(fd, termios.TCSADRAIN, old_settings)

        return ret

    @staticmethod
    def clear(lin_start=1, lin_end=1, attr=0):
        """
        Limpeza de linhas.

        :param lin_start:  Linha inicial utilizada na limpeza,
        :param lin_end: Linha final utilizada na limpeza,
        :param attr: limpa da posicao informada ate o final da tela se attr = 2 ou limpa da posicao
        informada ate o inicio da tela se attr = 1,
        :return: N/A.
        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        # Ajusta o atributo
        if 0 > attr < 2:
            attr = 0

        control = '\033[{0:d};{1:d}H\033[{2:d}J'
        print control.format(lin_start, lin_end, attr)

        return None

    @staticmethod
    def erase_line(*lines):
        """
        Apaga a linha atual.

        :param lines: numeros das linhas a serem apagadas (separadas por virgula)
        :return: quantidade de linhas apagadas.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        for line in lines:
            if line:
                print '\033[%d;1H\033[2K' % line
            else:
                print '\033[2K'

        result = len(lines)

        return result

    def beep(self):
        """
        Soa um ´beep´ (no terminal).

        :return: Som de ´beep´

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        print self.beepsound

    @staticmethod
    def quit():
        """
        Para a execucao do modulo terminal.

        :return: N/A.
        """

        print '\033[2J\033[0m\033[1;1f'  # ´Reset´ das cores
        exit(0)

    def menu(self, *args):
        """
        Monta o menu principal.

        :param args: Lista de opcoes para ser utilizada na montagem do menu,
        :return: numero da opcao selecionada.
        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        __CRLF = 13  # ´Enter´
        __ESCAPE = 27  # ´Escape´
        __CSICTRL = 91  # Caracter de controle do movimento do cursor
        __LOPTION = '{0:1d}. {1:<80s}'
        __OPCAO_FIM = 'FIM'  # Sempre a ultima opcao do menu principal
        __MENU_TITLE = '-  MENU PRINCIPAL  -'.center(self.max_col - 3)

        # Cores
        color_title = self.fg_cyan,  # Azul claro
        color_option = self.fg_white,  # Branco
        color_select = self.reverse, self.fg_white  # Preto/branco
        color_info = self.reverse, self.fg_cyan  # Preto/azul claro

        # Menu principal
        self.clear(2)
        self.say(2, 1, __MENU_TITLE, *color_title)
        self.say(3, 1, '--------------------', *color_title)

        # Monta as opcoes do menu
        linestart = 4  # Primeira linha de opcoes do menu
        lin = 0
        menuopt = []  # Opcoes do menu

        for option in args:
            try:
                option = str(option)
            except:
                option = unicodedata.normalize('NFKD', option).encode('ascii', 'ignore')

            menuopt.append(option)
            nlin = linestart + lin
            self.say(nlin, 1, __LOPTION.format(lin + 1, option), *color_option)
            lin += 1

        # Acrescenta item "FIM" ao menu
        menuopt.append(__OPCAO_FIM)
        self.say(linestart + lin, 1, __LOPTION.format(lin + 1, __OPCAO_FIM))
        lin += linestart + 1
        self.say(lin, 1, '-' * 20, *color_title)

        lin += 1
        self.say(lin, 1, '           Opcao [ ]', *color_info)

        # Menu marcado na primeira linha
        linemenu = __LOPTION.format(1, menuopt[0])
        self.say(linestart, 1, linemenu, *color_select)

        fd = sys.stdin.fileno()
        old_settings = termios.tcgetattr(fd)
        tty.setraw(fd)

        ret = ''
        lineoption = lin
        linestop = lin - 2
        lin = linestart

        while True:
            self.say(lineoption, 19, str(lin - linestart + 1))  # Mostra qual a opcao
            print '\033[{0:d};19f'.format(lineoption - 1)  # Posiciona o cursor
            r = sys.stdin.read(1)  # Aguarda a digitacao

            if ord(r) == __ESCAPE:
                r = sys.stdin.read(1)
                if ord(r) == __CSICTRL:
                    r = sys.stdin.read(1)

            if ord(r) == __CRLF:  # Antigo curses.KEY_ENTER
                ret = str(lin - linestart + 1)
                break
            elif r in ('A', 'D'):  # Cursor sobe ou esquerda
                if lin == linestart:  # Se igual a primeira linha do menu
                    continue
                elif lin == linestop:  # Se igual a ultima linha do menu
                    nlin = lin - linestart
                    linemenu = __LOPTION.format(nlin + 1, __OPCAO_FIM)
                else:
                    nlin = lin - linestart + 1
                    linemenu = __LOPTION.format(nlin, menuopt[nlin - 1])

                self.say(lin, 1, linemenu, *color_option)

                ret = ' '
                lin -= 1
                nlin = lin - linestart
                linemenu = __LOPTION.format(nlin + 1, menuopt[nlin])

                self.say(lin, 1, linemenu, *color_select)
            elif r in ('B', 'C'):  # Cursor desce ou direita
                if lin == linestop:  # Se igual a ultima linha do menu
                    continue

                nlin = lin - linestart + 1
                linemenu = __LOPTION.format(nlin, menuopt[nlin - 1])
                self.say(lin, 1, linemenu, *color_option)  # Linha atual

                lin += 1
                nlin += 1
                if nlin == linestop:  # Ultima linha
                    linemenu = __LOPTION.format(nlin, __OPCAO_FIM)
                    self.say(lin, 1, linemenu, *color_select)
                    continue

                linemenu = __LOPTION.format(nlin, menuopt[nlin - 1])
                self.say(lin, 1, linemenu, *color_select)  # Proxima linha do menu
                ret = ' '
            else:
                ret = r
                if ret.isdigit() and int(ret):
                    nlin = int(ret) + linestart - 1

                    if 0 < nlin <= linestop:
                        # Linha atual
                        nlin = lin - linestart + 1
                        linemenu = __LOPTION.format(nlin, menuopt[nlin - 1])
                        self.say(lin, 1, linemenu, *color_option)

                        # Linha selecionada
                        lin = int(ret) + linestart - 1
                        nlin = int(ret)
                        linemenu = __LOPTION.format(nlin, menuopt[nlin - 1])
                        self.say(lin, 1, linemenu, *color_select)
                    else:
                        self.beep()  # Sinal sonoro
                else:
                    ret = ' '

                if self.autoconfirm:
                    break

        termios.tcsetattr(fd, termios.TCSADRAIN, old_settings)

        return int(ret)

if __name__ == '__main__':
    TelnetEmulation()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

