# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2015 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2015, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv, fields
from functools import wraps
from openerp.tools.translate import _

# noinspection PyAbstractClass,PyPep8Naming
class estoque_movimento_origem(osv.Model):
    """
    Identifica a origem do movimento para operacoes de entrada ou saida.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'estoque.movimento.origem'  # estmog
    _inherit = 'estoque.movimento.origem'  # estoque/model/movimento_origem_data

    def get_movimento_origem(self, cr, user, ids, context=None):
        """
        Retorna lista com o codigo e o nome das origens de movimento.

        :param cr: cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID das origens de movimento
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type ids: long|list[long]
        :type context: dict
        :return: lista com o codigo e descricao das origens de movimento.
        :rtype: list[tuple]

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        args = [('id', 'in', ids if isinstance(ids, list) else [ids])]
        ids = self.search(cr, user, args, context=context)

        return map(lambda m: (m.get('estmog_codigo'), m.get('name')),
                   self.read(cr, user, ids, ['estmog_codigo', 'name'], context=context))

    # ___________________________ Decorators _____________________________
    # noinspection PyMethodMayBeStatic,PyMethodParameters
    def wrapper_estmog_codigo_upper(function):
        """
        Codigos de origem sempre sao grifados em CAIXA ALTA.

        :param function: funcao a ser encapsulada
        :type function: object
        :return: objeto/funcao tratado
        :rtype: object

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        # noinspection PyCallingNonCallable
        def inner(*args, **kwargs):
            for arg in args:
                if isinstance(arg, dict) and 'estmog_codigo' in arg:
                    arg['estmog_codigo'] = arg['estmog_codigo'].upper()

            return function(*args, **kwargs)

        return inner

    def wrapper_estmog_mantem_origem_padrao(function):
        """
        Deleta uma origem se ela nao for uma origem padrao do sistema
        :param function: funcao a ser encapsulada
        :type function: object
        :return object

        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        # noinspection PyCallingNonCallable
        @wraps(function)
        def mantem_origem_padrao(self, cr, uid, ids, *args, **kwargs):
            origem = self.get_movimento_origem(cr, uid, ids, **kwargs)
            if origem and any(map(lambda i: i in 'NCRSIP', dict(origem).keys())):
                raise osv.except_osv(_('Atencao'), _('Nao e permitido excluir/editar uma origem '
                                                     'padrao do sistema'))
            return function(self, cr, uid, ids, *args, **kwargs)

        return mantem_origem_padrao

    # ___________________________ ORM ____________________________________
    # noinspection PyArgumentList
    @wrapper_estmog_codigo_upper  # Codigos de origem sempre sao grifados em CAIXA ALTA
    def create(self, cr, user, vals, context=None):
        """
        Sobrescreve o metodo.

        :param cr: cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param vals: dicionario contendo os dados do registro
        :param context: um dicionario padrao para valores contextuais
        :type cr: cursor
        :type user: int
        :type vals: dict
        :type context: dict
        :return: ID do novo registro criado
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br
        """

        context = context and context.copy() or {}

        return super(estoque_movimento_origem, self).create(cr, user, vals, context)

    # noinspection PyArgumentList
    @wrapper_estmog_codigo_upper  # Codigos de origem sempre sao grifados em CAIXA ALTA
    @wrapper_estmog_mantem_origem_padrao  # Matem as origens padroes do sistema
    def write(self, cr, user, ids, vals, context=None):
        """
        Sobrescreve o metodo.

        :param cr: cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID das origens de movimento
        :param vals: dicionario contendo os dados do registro
        :param context: um dicionario padrao para valores contextuais
        :type cr: cursor
        :type user: int
        :type ids: long|list[long]
        :type vals: dict
        :type context:dict
        :return: verdadeiro se o registro foi alterado
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br
        """

        context = context and context.copy() or {}

        return super(estoque_movimento_origem, self).write(cr, user, ids, vals, context)

    # noinspection PyArgumentList
    @wrapper_estmog_mantem_origem_padrao
    def unlink(self, cr, uid, ids, context=None):
        """
        Sobrescreve o metodo unlink.

        :param cr: cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID das origens de movimento
        :param context: um dicionario padrao para valores contextuais
        :type cr: cursor
        :type uid: int
        :type ids: long|list[long]
        :type context:dict
        :return: verdadeiro se o registro foi alterado
        :rtype: bool

        :author: Thiago Abreu <thiago.rodrigues[at]siginformatica.com.br
        """

        context = context and context.copy() or {}
        return super(estoque_movimento_origem, self).unlink(cr, uid, ids, context=context)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
