# -*- coding: utf-8 -*-

from openerp.osv import osv
from openerp.tools.translate import _


class estoque_movimento_itens(osv.Model):
    """
    PREFIXO: ESTMVI
    complemento do movimento do estoque

    """
    _inherit = 'estoque.movimento.itens'

    # noinspection PyMethodMayBeStatic
    def onchange_volume(self, cr, uid, ids, quantidade, volume, unitario, desconto, context=None):
        """
        Calcula o valor total do item.

        :param cr: cursor do banco de dados
        :param uid: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do formulario de inclusao de item de movimento
        :param quantidade: a quantidade de itens
        :param volume: o volume em metros quadrados ou cubicos
        :param unitario: o valor unitario do item
        :param desconto: o valor do desconto aplicado
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type ids: list
        :type quantidade: float
        :type volume: float
        :type unitario: float
        :type desconto: float
        :type context: dict
        :return: o valor total do item.
        :rtype: dict
        """
        context = context and context.copy() or {}

        return dict(value=dict(vr_total_item=unitario * (volume or quantidade) - desconto))

    @staticmethod
    def estmvi_valida_disponibilidade_saldo(saldo, dados_item):
        """
        Verifica a disponibilidade do saldo do item

        :param saldo: saldo do item
        :param brw_item: browse do item da nota (estoque.movimento.itens)
        :type saldo: float
        :type brw_item: browser
        :rtype: bool
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        if 0 > saldo - dados_item.get('estmvi_volume') or 0:
            return False

        return True

    @staticmethod
    def estmvi_get_mensagem_saldo_indisp(produto_nome, local_nome, dados_item, saldo):
        """
        Retorna a mensagem de saldo indisponivel

        :param produto_nome: Nome do produto
        :param local_nome: Nome do local informado para o produto
        :param brw_item: browse do item da nota (estoque.movimento.itens)
        :param saldo: saldo do item
        :type produto_nome: str
        :type local_nome: str
        :type brw_item: browser
        :type saldo: float
        :rtype: str
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        msg = '* Saldo indisponivel para o produto: (%s) no local: (%s) - ' \
              'qtde. pedida: %.3f - disponivel: %.3f;' % (
                  produto_nome, local_nome, dados_item.get('estmvi_volume') or 0, saldo)
        return msg

    @staticmethod
    def estmvi_get_query_item_requisitado(grade_ids, company_id, esboco=False):
        """
        Retorna a query para verificar a quantidade reservada do item para as notas fiscais de saida
        :param grade_ids: Os ids das grades para ser verificada
        :param company_id:
        :type grade_ids: list
        :type company_id: int
        :rtype: str
        :author: Thiago Abreu Rodrigues <thiago.rodrigues[at]siginformatica.com.br>
        """
        if esboco:
            _status = "('3', 'C', 'D' )"
        else:
            _status = "( '1', '3', 'C', 'D' )"
        str_grade_ids = '('+','.join(str(e) for e in grade_ids)+')'
        # state da nf - 1-Esboço, 3-Faturada, C-Cancelada, D-Denegada
        _sql = """
                SELECT grade_id,
                       local_id,
                       Sum(quantidade) AS quantidade
                FROM   (SELECT grade_id,
                               local_id,
                               CASE
                                 WHEN COALESCE(estmvi_volume, 0.0) > 0 THEN estmvi_volume
                                 ELSE quantidade
                               END AS quantidade
                        FROM   estoque_movimento_itens i
                               LEFT JOIN fiscal_nf_capa c
                                      ON c.id = i.nf_id
                        WHERE  grade_id IN {grade_ids}
                               AND state NOT IN {status}
                               AND tipo = 'S'
                               AND company_id = {company_id}) AS TMP
                GROUP  BY grade_id,
                          local_id
        """.format(grade_ids=str_grade_ids, status=_status, company_id=company_id)
        return _sql

    def estmvi_onchange_quantidade_propria(self, cr, user, ids, quantidade, unitario, desconto,
                                           embalagem, context=None, comprimento=None, largura=None,
                                           espessura=None):
        """
        Calcula o volume em funcao da quantidade, comprimento, largura e espessura do item.

        :param cr: cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do formulario de inclusao de item de movimento
        :param quantidade: a quantidade de itens
        :param unitario: o valor unitario do item
        :param desconto: o valor do desconto rateado (proporcional ao desconto na capa)
        :param embalagem: o ID da embalagem
        :param context: um dicionario padrao para valores contextuais
        :param comprimento: o comprimento em metros
        :param largura: a largura em centimetros
        :param espessura: a espessura em centimetros
        :type cr: object
        :type user: int
        :type ids: list
        :type quantidade: float
        :type unitario: float
        :type desconto: float
        :type embalagem: long
        :type context: dict
        :type comprimento: float
        :type largura: float
        :type espessura: float
        :return: o volume.
        :rtype: float

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}

        # Dados da embalagem
        brw_embalagem = self.pool.get('estoque.produto.embalagem').browse(
            cr, user, embalagem, context)

        # Calculo do volume em funcao da metragem quadrada ou cubica
        sigla = embalagem and brw_embalagem.embalagem_id.sigla
        if sigla in ('M2', 'M3', 'ML'):
            volume = self.pool.get('madeireira.utils').madutl_calcula_volume(
                quantidade, comprimento, espessura, largura)
            result = dict(value=dict(estmvi_volume=volume))
        else:
            volume = 0
            result = dict(value=dict(estmvi_volume=volume, vr_total_item=quantidade * unitario))

        return result

    def estmvi_onchange_comprimento(self, cr, user, ids, item, quantidade, comprimento, largura,
                                    espessura, unitario, context=None):
        """
        Calcula o volume em funcao da quantidade, comprimento, largura e espessura do item. Tambem
        Calcula o valor unitario se o item tem variacao por comprimento.

        :param cr: cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do formulario de inclusao de item de movimento
        :param item: o ID do item
        :param quantidade: a quantidade de itens
        :param comprimento: o comprimento em metros
        :param largura: a largura em centimetros
        :param espessura: a espessura em centimetros
        :param unitario: o valor unitario do item
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type item: long
        :type quantidade: float
        :type comprimento: float
        :type largura: float
        :type espessura: float
        :type unitario: float
        :type context: dict
        :return: o volume.
        :rtype: float

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        if not item:
            return dict()

        # Dados do item
        brw_item = self.pool.get('estoque.produto').browse(cr, user, item, context)

        # Recupera o fator da variacao para ajuste do valor unitario
        msg = None
        if brw_item.estpro_variacao == 'C':  # Variacao: ´C´ por comprimento e ´E´ por espessura
            fator = self.pool.get('madeireira.utils').madutl_get_fator_variacao(
                cr, user, item, comprimento,  context)
            if comprimento and not fator:
                msg = _('Este item utiliza variacao de comprimento. O comprimento %s nao se '
                        'encontra cadastrado para o item %s') % (comprimento, brw_item.name)
                comprimento = 0
            else:  # Novo valor unitario para o item
                unitario = unitario * fator if comprimento else unitario

        # Calcula o volume
        volume = self.pool.get('madeireira.utils').madutl_calcula_volume(
            quantidade, comprimento, largura, espessura)

        result = dict(value=dict(estmvi_volume=volume, vr_unitario=unitario,
                                 estmvi_comprimento=comprimento))
        if msg:
            result = dict(result, warning=dict(title=_('Aviso.'), message=msg))

        return result

    def estmvi_onchange_espessura(self, cr, user, ids, item, quantidade, comprimento, largura,
                                  espessura, unitario, context=None):
        """
        Calcula o volume em funcao da quantidade, comprimento, largura e espessura do item. Tambem
        Calcula o valor unitario se o item tem variacao por espessura.

        :param cr: cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do formulario de inclusao de item de movimento
        :param item: o ID do item
        :param quantidade: a quantidade de itens
        :param comprimento: o comprimento em metros
        :param largura: a largura em centimetros
        :param espessura: a espessura em centimetros
        :param unitario: o valor unitario do item
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type item: long
        :type quantidade: float
        :type comprimento: float
        :type largura: float
        :type espessura: float
        :type unitario: float
        :type context: dict
        :return: o volume.
        :rtype: float

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}
        if not item:
            return dict()

        # Dados do item
        brw_item = self.pool.get('estoque.produto').browse(cr, user, item, context)

        # Recupera o fator da variacao para ajuste do valor unitario
        msg = None
        if brw_item.estpro_variacao in 'E':  # Variacao: ´C´ por comprimento ou ´E´ por espessura
            fator = self.pool.get('madeireira.utils').madutl_get_fator_variacao(
                cr, user, item, espessura, context)
            if espessura and not fator:
                msg = _('Este item utiliza variacao de espessura. A espessura %s nao se '
                        'encontra cadastrado para o item %s') % (espessura, brw_item.name)
                espessura = 0
            else:  # Novo valor unitario para o item
                unitario = unitario * fator if espessura else unitario

        # Calcula o volume
        volume = self.pool.get('madeireira.utils').madutl_calcula_volume(
            quantidade, comprimento, largura, espessura)

        result = dict(value=dict(estmvi_volume=volume, vr_unitario=unitario,
                                 estmvi_espessura=espessura))
        if msg:
            result = dict(result, warning=dict(title=_('Aviso.'), message=msg))

        return result

    def estmvi_onchange_largura(self, cr, user, ids, quantidade, comprimento, largura, espessura,
                                unitario, context=None):
        """
        Calcula o volume em funcao da quantidade, comprimento, largura e espessura do item

        :param cr: cursor do banco de dados
        :param user: o ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com o ID do formulario de inclusao de item de movimento
        :param quantidade: a quantidade de itens
        :param comprimento: o comprimento em metros
        :param largura: a largura em centimetros
        :param espessura: a espessura em centimetros
        :param unitario: o valor unitario do item
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type ids: list
        :type quantidade: float
        :type comprimento: float
        :type largura: float
        :type espessura: float
        :type unitario: float
        :type context: dict
        :return: o volume.
        :rtype: float

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}

        # Calcula o volume
        volume = self.pool.get('madeireira.utils').madutl_calcula_volume(
            quantidade, comprimento, largura, espessura)

        result = dict(value=dict(estmvi_volume=volume))

        return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
