# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
from openerp.tools.translate import _


class pcp_simulacao_precos(osv.TransientModel):
    """
    Simula precos de venda, encargos e margem de lucro.
    """

    _name = 'pcp.wizard.simulacao.precos'

    def create(self, cr, uid, vals, context=None):
        """
        Cria a capa da simulacao.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param vals: {dict} dicionario com os valores dos campos,
        @param context: {dict} um dicionario padrao para valores contextuais,
        @return: {int} ID da simulacao que foi criada.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        vals.update(self._dados_capa_get(cr, uid, [], vals.get('capa_id', False), context=context))

        result = super(pcp_simulacao_precos, self).create(cr, uid, vals, context)

        return result

    def write(self, cr, uid, ids, vals, context=None):
        """
        Altera os dados da capa da simulacao.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param vals: {dict} dicionario com os valores dos campos,
        @param context: {dict} um dicionario padrao para valores contextuais,
        @return: {int} ID da simulacao que foi criada.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        vals.update(self._dados_capa_get(cr, uid, ids, vals.get('capa_id', False), context=context))

        result = super(pcp_simulacao_precos, self).write(cr, uid, ids, vals, context)

        return result

    def _get_custo_fixo(self, cr, uid, context=None):
        context = context and context.copy() or {}
        obj_estoque_parametro = self.pool.get('estoque.parametro')
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name, context=context)
        parametro_id = obj_estoque_parametro.search(cr, uid, [('company_id', '=', company_id)], context=context)
        custo_fixo = obj_estoque_parametro.read(cr, uid, parametro_id[0], ['custo_fixo'], context=context)
        return custo_fixo.get('custo_fixo', 0)

    def _get_comissao(self, cr, uid, context=None):
        """
        Retorna o percentual da comissao baseado no parametro do estoque

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @return: {float} valor percentual de comissao (padrao).

        @see: comissoes (estoque.parametro)
        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        # ___________________________ Objects ________________________________
        obj_estoque_parametro = self.pool.get('estoque.parametro')

        result = 0
        company_id = self.pool.get('adm.empresa.objeto').getEmpresa(cr, uid, self._name, context=context)
        parametro_ids = obj_estoque_parametro.search(cr, uid, [('company_id', '=', company_id)], context=context)
        campos = ['comissoes']
        dados_comissao = obj_estoque_parametro.read(cr, uid, parametro_ids, campos, context=context)

        if dados_comissao and isinstance(dados_comissao, list):
            dados_comissao = dados_comissao[0]

        if dados_comissao and isinstance(dados_comissao, dict):
            result = dados_comissao.get('comissoes', 0)

        return result

    def _get_preco_custo_para_venda(self, cr, uid, produto_id, context=None):
        """
        Recupera o preco de custo para venda baseado no parametro do estoque.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param produto_id: {int} ID do produto,
        @return: {float} o preco de custo para venda.

        @see estpar_custo_para_precovenda (estoque.parametro)
        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        # ___________________________ Objects ________________________________
        obj_adm_utils = self.pool.get('adm.utils')
        obj_estoque_preco_custo = self.pool.get('estoque.preco.custo')

        result = 0.0
        dados_parametro = obj_adm_utils.admutl_get_parametros(
            cr, uid, 'estoque', ['estpar_custo_para_precovenda'])
        parametro = dados_parametro.get('estpar_custo_para_precovenda', 'U')
        custo_tipo = {'U': 'ultimo_custo', 'M': 'custo_medio_contabil', 'R': 'custo_reposicao',
                      'C': 'custo_corrigido'}
        args = [('produto_id', '=', produto_id)]
        estoque_preco_custo_ids = obj_estoque_preco_custo.search(cr, uid, args, context=context)
        campos = custo_tipo.values()
        dados_preco_custo = obj_estoque_preco_custo.read(cr, uid, estoque_preco_custo_ids, campos, context=context)

        if dados_preco_custo and isinstance(dados_preco_custo, list):
            dados_preco_custo = dados_preco_custo[0]

        if dados_preco_custo and isinstance(dados_preco_custo, dict):
            result = dados_preco_custo.get(custo_tipo.get(parametro, 'U'), 0.0)

        return result

    def _get_custo_total(self, cr, uid, ficha_id, context=None):
        """
        Calcula o custo total do produto.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ficha_id: {int} ID do produto.
        @return: {float} o valor do custo total do produto.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        # ___________________________ Objects ________________________________
        obj_pcp_ficha_tecnica_capa = self.pool.get('pcp.ficha.tecnica.capa')
        obj_pcp_ficha_tecnica_itens = self.pool.get('pcp.ficha.tecnica.itens')

        item_ids = []  # Lista dos itens
        campos = ['pcpftc_ficha_tecnica_itens_ids']
        dados_ficha_tecnica = obj_pcp_ficha_tecnica_capa.read(cr, uid, ficha_id, campos, context=context)

        if dados_ficha_tecnica and isinstance(dados_ficha_tecnica, list):
            dados_ficha_tecnica = dados_ficha_tecnica[0]

        if dados_ficha_tecnica and isinstance(dados_ficha_tecnica, dict):
            item_ids = dados_ficha_tecnica.get('pcpftc_ficha_tecnica_itens_ids', [])

        campos = ['pcpfti_custo_total']
        dados_ficha_tecnica_itens = obj_pcp_ficha_tecnica_itens.read(cr, uid, item_ids, campos, context=context)

        custo_total = 0.0
        if dados_ficha_tecnica_itens and isinstance(dados_ficha_tecnica_itens, list):
            custo_total = sum(map(lambda item: item.get('pcpfti_custo_total', 0.0),
                                  dados_ficha_tecnica_itens))

        if dados_ficha_tecnica_itens and isinstance(dados_ficha_tecnica_itens, dict):
            custo_total = dados_ficha_tecnica_itens.get('pcpfti_custo_total', 0.0)

        return custo_total

    def _calcular_lucro(self, cr, uid, produto_id, preco_venda, encargos, context=None):
        """
        Calcula a margem de lucro.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param produto_id: {int} ID do produto,
        @param preco_venda: {float} preco de venda do produto,
        @param encargos: {float} somatorio dos encargos. %preco_fixo + %comissao + %total dos
        encargos.
        @return: {float} margem de lucro.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        # ___________________________ Objects ________________________________
        obj_estoque_utils = self.pool.get('estoque.utils')

        # Preco de custo baseado no preco de venda
        preco_custo = self._get_preco_custo_para_venda(cr, uid, produto_id, context=context)

        result = obj_estoque_utils.estutl_calcular_margem(preco_venda, encargos, preco_custo)
        result = 0.0 if result < 0 else result

        return result

    def _criar_encargos(self, cr, uid, ids, produto_id, preco_venda, context=None):
        """
        Cria os encargos para simulacao baseado nos encargos dos produtos.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param produto_id: {int} o ID do produto,
        @param preco_venda: {float} o preco de venda.
        @return: {list} lista com o ID dos encargos que foram criados.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_fiscal_tributacao_encargos = self.pool.get('fiscal.tributacao.encargos')
        obj_pcp_wizard_simulacao_precos_encargos = self.pool.get('pcp.wizard.simulacao.precos.encargos')

        result = []
        args = [('produto_id', '=', produto_id), ('tipo_tributacao', '=', 'T')]
        tributacao_encargo_ids = obj_fiscal_tributacao_encargos.search(cr, uid, args, context=context)
        args = [('encargo_capa_id', 'in', ids)]
        encargo_ids = obj_pcp_wizard_simulacao_precos_encargos.search(cr, uid, args, context=context)
        self.pool.get('pcp.wizard.simulacao.precos.encargos').unlink(cr, uid, encargo_ids)

        for encargo in obj_fiscal_tributacao_encargos.browse(cr, uid, tributacao_encargo_ids):
            aliquota = encargo.tributacao_id.aliquota
            base = encargo.tributacao_id.base
            encargo = encargo.encargo_id.name
            valor = aliquota * base * preco_venda / 10000
            values = {'aliquota': aliquota, 'base': base, 'encargo': encargo,
                      'preco_venda': preco_venda, 'valor': valor}

            # Cria os encargos
            result.append((0, 0, values))

        return result

    def _criar_insumos(self, cr, uid, ids, ficha_id, context=None):
        """
        Cria os insumos para simulacao baseado no produto.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param ficha_id: {int} o ID da ficha tecnica do produto,
        @return: {list} lista com o ID dos encargos que foram criados.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_pcp_ficha_tecnica_itens = self.pool.get('pcp.ficha.tecnica.itens')
        obj_pcp_wizard_simulacao_precos_insumos = self.pool.get('pcp.wizard.simulacao.precos.insumos')

        result = []
        args = [('pcpfti_capa_id', '=', ficha_id)]
        insumo_ids = obj_pcp_ficha_tecnica_itens.search(cr, uid, args, context=context)
        args = [('simulacao_capa_id', 'in', ids)]
        encargo_ids = obj_pcp_wizard_simulacao_precos_insumos.search(cr, uid, args, context=context)
        obj_pcp_wizard_simulacao_precos_insumos.unlink(cr, uid, encargo_ids)

        for insumo in obj_pcp_ficha_tecnica_itens.browse(cr, uid, insumo_ids):
            produto_id = insumo.pcpfti_produto_id.id
            quantidade = insumo.pcpfti_quantidade
            custo_total = insumo.pcpfti_custo_total
            values = {'produto_id': produto_id, 'quantidade': quantidade,
                      'custo_total': custo_total}

            # Cria os insumos
            result.append((0, 0, values))

        return result

    def onchange_ficha(self, cr, uid, ids, ficha_id, context=None):
        """
        Recupera os dados da ficha tecnica e demais dados do produto que serao utilizados na
        simulacao.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param ficha_id: {int} ID da ficha.
        @return: {dict} dicionario com os valores da ficha.
        """

        context = context and context.copy() or {}
        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        result = {'value': self._dados_capa_get(cr, uid, ids, ficha_id, context=context)}

        return result

    def _dados_capa_get(self, cr, uid, ids, ficha_id, context=None):
        """
        Recupera os dados da ficha do produto necessarios a simulacao

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param ficha_id: {int} ID da ficha.
        @return: {dict} dicionario com os valores da ficha.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        if ids and not ficha_id:
            campos = ['capa_id']
            dados_ficha_tecnica = self.read(cr, uid, ids, campos, context=context, load='_classic_write')

            if dados_ficha_tecnica and isinstance(dados_ficha_tecnica, list):
                dados_ficha_tecnica = dados_ficha_tecnica[0]

            if dados_ficha_tecnica and isinstance(dados_ficha_tecnica, dict):
                ficha_id = dados_ficha_tecnica.get('capa_id', False)

        result = self._dados_ficha_get(cr, uid, ficha_id, context=context)
        produto_id = result.get('produto_id', False)
        grade_id = result.get('grade_id') or []

        # Dados do produto
        result.update(self._dados_produto_get(cr, uid, produto_id, grade_id))  # Dados do produto
        preco_venda = result.get('preco_venda', 0.0)

        # Custos
        custo_fixo_porcentagem = self._get_custo_fixo(cr, uid, context=context)
        custo_fixo_valor = preco_venda * custo_fixo_porcentagem / 100.0
        comissao = self._get_comissao(cr, uid, context=context)
        comissao_valor = preco_venda * comissao / 100.0

        # Encargos
        encargo_ids = self._criar_encargos(cr, uid, ids, produto_id, preco_venda, context)
        encargo_valor = sum(map(lambda encargo: encargo[2].get('valor'), encargo_ids))
        encargo_porcentagem = sum(map(lambda encargo: encargo[2].get('aliquota'), encargo_ids))
        encargos = custo_fixo_porcentagem + comissao + encargo_porcentagem

        # Insumos
        insumo_ids = self._criar_insumos(cr, uid, ids, ficha_id, context)

        result['custo_total'] = self._get_custo_total(cr, uid, ficha_id, context=context)  # Custo total do produto
        result['preco_custo'] = self._get_preco_custo_para_venda(cr, uid, produto_id, context=context)
        result['custo_fixo_porcentagem'] = custo_fixo_porcentagem
        result['custo_fixo_valor'] = custo_fixo_valor
        result['comissao'] = comissao
        result['comissao_valor'] = comissao_valor
        result['encargo_ids'] = encargo_ids
        result['insumo_ids'] = insumo_ids
        result['encargo_valor'] = encargo_valor
        result['encargo_porcentagem'] = encargo_porcentagem
        result['encargo_total'] = encargo_valor + custo_fixo_valor + comissao_valor
        result['lucro_porcentagem'] = self._calcular_lucro(
            cr, uid, produto_id, preco_venda, encargos, context=context)

        return result

    def _calcular_encargo_total(self, cr, uid, encargo_ids, context=None):
        """
        Calcula o somatorio das aliquotas dos encargos ou o somatorio dos valores dos encargos
        aplicados ao produto de acordo com o contexto.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param encargo_ids: {list} a lista dos IDs referentes os encargos do produto.
        @return: {float} o somatorio das aliquotas ou se o contexto for ´´valor´´, sera retornado o
        somatorio dos valores aplicados.
        @attention: contexto ´´encargo´´ = "valor"

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_pcp_wizard_simulacao_precos_encargos = self.pool.get('pcp.wizard.simulacao.precos.encargos')

        result = 0.0
        campos = ['base', 'aliquota', 'valor']
        dados_simulacao_precos_encargos = obj_pcp_wizard_simulacao_precos_encargos.read(
            cr, uid, encargo_ids, campos, context=context)

        if dados_simulacao_precos_encargos and isinstance(dados_simulacao_precos_encargos, list):
            if context.get('encargo') == 'valor':
                result = sum(map(lambda encargo: encargo.get('valor', 0.0),
                                 dados_simulacao_precos_encargos))
            else:
                result = sum(map(
                    lambda encargo: encargo.get('base', 0.0) * encargo.get('aliquota', 0.0) / 100,
                    dados_simulacao_precos_encargos))

        if dados_simulacao_precos_encargos and isinstance(dados_simulacao_precos_encargos, dict):
            if context.get('encargo', 'valor'):
                result = dados_simulacao_precos_encargos.get('valor', 0.0)
            else:
                base = dados_simulacao_precos_encargos.get('base', 0.0)
                aliquota = dados_simulacao_precos_encargos.get('aliquota', 0.0)
                result = base * aliquota / 100

        return result

    def _dados_ficha_get(self, cr, uid, ficha_id, context=None):
        """
        Recupera os dados da ficha do produto necessarios a simulacao.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ficha_id: {int} ID da ficha.
        @return: {dict} dicionario com os dados da ficha utilizapara simulacao.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        # ___________________________ Objects ________________________________
        obj_pcp_ficha_tecnica_capa = self.pool.get('pcp.ficha.tecnica.capa')

        result = {'capa_id': ficha_id}

        # Dados da ficha tecnica
        campos = ['pcpftc_produto_id', 'pcpftc_grade_id', 'pcpftc_produto_peso_embalagem_id',
                  'pcpftc_quantidade_peso']
        dados_ficha_tecnica = obj_pcp_ficha_tecnica_capa.read(
            cr, uid, ficha_id, campos, context=context, load='_classic_write')

        if dados_ficha_tecnica and isinstance(dados_ficha_tecnica, list):
            dados_ficha_tecnica = dados_ficha_tecnica[0]

        if dados_ficha_tecnica and isinstance(dados_ficha_tecnica, dict):
            produto_id = dados_ficha_tecnica.get('pcpftc_produto_id', False)
            result['produto_id'] = produto_id
            grade_id = dados_ficha_tecnica.get('pcpftc_grade_id', False)
            result['grade_id'] = grade_id
            produto_embalagem_id = dados_ficha_tecnica.get('pcpftc_produto_peso_embalagem_id', False)
            result['produto_embalagem_id'] = produto_embalagem_id
            result['quantidade'] = dados_ficha_tecnica.get('pcpftc_quantidade_peso', 0.0)

        return result

    def _dados_produto_get(self, cr, uid, produto_id, grade_id):
        """
        Recupera os dados do produto necessarios a simulacao.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param produto_id: {int} ID do produto.
        @return: {dict} dicionario com os dados da ficha utilizapara simulacao.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        # ___________________________ Objects ________________________________
        obj_estoque_produto = self.pool.get('estoque.produto')
        obj_estoque_preco_venda = self.pool.get('estoque.preco.venda')

        result = {}

        # Margens
        margens = obj_estoque_produto.estpro_get_margens(cr, uid, produto_id)
        result['margem_minima'] = margens.get(produto_id, {}).get('margem_minima', 0.0)
        result['margem_ideal'] = margens.get(produto_id, {}).get('margem_ideal', 0.0)
        result['margem_bruta'] = margens.get(produto_id, {}).get('margem_bruta', 0.0)

        # Preco de venda
        preco_venda = obj_estoque_preco_venda.get_preco_venda(
            cr, uid, produto_id, epc_grade_ids=grade_id)
        dados_preco_venda = preco_venda.get(produto_id) or {}
        valor_preco_venda = dados_preco_venda.get(grade_id) or dados_preco_venda.get(
            'preco_venda')
        result['preco_venda'] = valor_preco_venda

        return result

    def onchange_preco_venda_simulacao(self, cr, uid, ids, ficha_id, produto_id, preco_venda,
                                       custo_fixo, comissao, context=None):
        """
        Calcula a margem de lucro em funcao do preco de venda da simulacao.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param ficha_id: {int} IFD da ficha tecnica do produto,
        @param produto_id: {int} o ID produto,
        @param preco_venda: {float} o preco de venda a ser utilizado para simulacao,
        @param custo_fixo: {float} o percentual do custo fixo,
        @param comissao: {float} o percentual da comissao
        @return: {dict} dicionario com os valores recalculados.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        if not preco_venda:
            raise osv.except_osv(_('Atencao!'), _('Preco de venda nao pode ser zero.'))

        if not ficha_id:
            msg = 'Para calcular os valores da simulacao, a ficha do produto deve ser especificada.'
            raise osv.except_osv(_('Atencao!'), _(msg))

        values = {'capa_id': ficha_id}

        # Atualiza os valores dos encargos
        custo_fixo_valor = preco_venda * custo_fixo / 100.0
        comissao_valor = preco_venda * comissao / 100.0
        values['custo_fixo_valor'] = custo_fixo_valor
        values['comissao_valor'] = comissao_valor
        encargo_ids = self._criar_encargos(cr, uid, ids, produto_id, preco_venda, context)
        encargo_valor = sum(map(lambda encargo: encargo[2].get('valor'), encargo_ids))
        encargo_porcentagem = sum(map(lambda encargo: encargo[2].get('aliquota'), encargo_ids))
        values['encargo_valor'] = encargo_valor
        values['encargo_porcentagem'] = encargo_porcentagem
        values['encargo_total'] = encargo_valor + custo_fixo_valor + comissao_valor
        values['encargo_ids'] = encargo_ids  # Atualiza!

        # Novo preco de venda
        values['preco_venda'] = preco_venda  # Preco de venda recebe o valor da simulacao

        # Margem de lucro
        encargos = encargo_porcentagem + custo_fixo + comissao

        # Margem para preco de venda
        estoque_parametros = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'estoque', 'estpar_margem_para_preco_venda')
        margem_preco_venda = estoque_parametros.get('estpar_margem_para_preco_venda')
        if margem_preco_venda != 'B':
            encargos = 0.0

        values['lucro_porcentagem'] = self._calcular_lucro(
            cr, uid, produto_id, preco_venda, encargos, context=context)

        result = {'value': values}

        return result

    def onchange_margem_lucro_simulacao(self, cr, uid, ids, ficha_id, produto_id, margem,
                                        custo_fixo, comissao, encargos, context=None):
        """
        Calcula o novo valor para o preco de venda do produto em funcao da margem de lucro
        informada.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param ficha_id: {int} ID da ficha tecnica do produto,
        @param produto_id: {int} ID do produto,
        @param margem: {float} percentual de lucro utilizado para simulacao,
        @param custo_fixo: {float} o custo fixo do produto,
        @param comissao: {float} percentual de comissao,
        @param encargos: {float} somatorio dos encargos do produto.
        @return: {dict} dicionario com os valores alterados em funcao da margem de lucro informada.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        # ___________________________ Objects ________________________________
        obj_estoque_utils = self.pool.get('estoque.utils')

        if not ficha_id:
            msg = 'Para calcular os valores da simulacao, a ficha do produto deve ser especificada.'
            raise osv.except_osv(_('Atencao!'), _(msg))

        values = {'capa_id': ficha_id}

        # Preco de custo baseado no parametro de estoque (estpar_custo_para_precovenda)
        preco_custo = self._get_preco_custo_para_venda(cr, uid, produto_id, context=context)

        # Novos valores em funcao do novo preco de venda
        encargo_total = custo_fixo + comissao + encargos

        # Margem para preco de venda
        estoque_parametros = self.pool.get('adm.utils').admutl_get_parametros(
            cr, uid, 'estoque', 'estpar_margem_para_preco_venda')
        margem_preco_venda = estoque_parametros.get('estpar_margem_para_preco_venda')
        if margem_preco_venda != 'B':
            encargo_total = 0.0

        novo_preco_venda = obj_estoque_utils.estutl_calcular_preco_de_venda(
            preco_custo, encargo_total, margem)

        # Se o novo preco de venda for zero, entao nao foi possivel calcular
        if novo_preco_venda:
            values['preco_venda'] = novo_preco_venda
            values['preco_custo'] = preco_custo
            custo_fixo_valor = novo_preco_venda * custo_fixo / 100.0
            comissao_valor = novo_preco_venda * comissao / 100.00
            values['custo_fixo_valor'] = custo_fixo_valor
            values['comissao_valor'] = comissao_valor

            # Atualiza os valores dos encargos em funcao do novo preco de venda
            encargo_ids = self._criar_encargos(cr, uid, ids, produto_id, novo_preco_venda, context)
            encargo_valor = sum(map(lambda encargo: encargo[2].get('valor'), encargo_ids))
            encargo_porcentagem = sum(map(lambda encargo: encargo[2].get('aliquota'), encargo_ids))

            values['encargo_porcentagem'] = encargo_porcentagem
            values['encargo_valor'] = encargo_valor
            values['encargo_total'] = encargo_valor + custo_fixo_valor + comissao_valor
            values['lucro_porcentagem'] = margem  # Lucro % recebe a nova margem de lucro
            values['encargo_ids'] = encargo_ids  # Atualiza os encargos!
            result = {'value': values}
        else:
            msg = _('Nao foi possivel calcular o preco de venda para o percentual de lucro '
                    'informado.')
            result = {'warning': {'title': _('Aviso.'), 'message': msg}}

        return result

    def button_preco_venda_atual(self, cr, uid, ids, context=None):
        """
        Recupera o preco de venda atual

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param context: {dict} um dicionario padrao para valores contextuais,
        @return: {boolean} verdadeiro se o valor do preco de venda atual foi recuperado.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_estoque_preco_venda = self.pool.get('estoque.preco.venda')

        preco_venda = 0.0
        campos = ['capa_id', 'produto_id', 'grade_id']
        dados_simulacao = self.read(cr, uid, ids, campos, context=context, load='_classic_write')

        if dados_simulacao and isinstance(dados_simulacao, list):
            dados_simulacao = dados_simulacao[0]

        if dados_simulacao and isinstance(dados_simulacao, dict):
            ficha_id = dados_simulacao.get('capa_id', False)
            produto_id = dados_simulacao.get('produto_id', False)
            grade_id = dados_simulacao.get('grade_id') or []

            # Recupera o preco de venda
            preco_venda = obj_estoque_preco_venda.get_preco_venda(
                cr, uid, produto_id, epc_grade_ids=grade_id)
            dados_preco_venda = preco_venda.get(produto_id) or {}
            preco_venda = dados_preco_venda.get(grade_id) or dados_preco_venda.get(
                'preco_venda')

            values = {'preco_venda_simulacao': preco_venda, 'capa_id': ficha_id}
            self.write(cr, uid, ids, values, context)

        return preco_venda

    def button_margem_ideal(self, cr, uid, ids, context=None):
        """
        Recupera o valor da margem ideal.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param context: {dict} um dicionario padrao para valores contextuais,
        @return: {boolean} verdadeiro se o valor da margem idel foi recuperado.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        margem_ideal = 0.0
        campos = ['capa_id', 'margem_ideal']
        dados_simulacao = self.read(cr, uid, ids, campos, context=context, load='_classic_write')

        if dados_simulacao and isinstance(dados_simulacao, list):
            dados_simulacao = dados_simulacao[0]

        if dados_simulacao and isinstance(dados_simulacao, dict):
            ficha_id = dados_simulacao.get('capa_id', False)
            margem_ideal = dados_simulacao.get('margem_ideal', 0.0)

            values = {'lucro_simulacao': margem_ideal, 'capa_id': ficha_id}
            self.write(cr, uid, ids, values, context)

        return margem_ideal

    def button_atualizar_preco_venda_custo(self, cr, uid, ids, context=None):
        """
        Ataliza o preco de venda e preco de custo na base do SIGERP/SIGGESTAO

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param context: {dict} um dicionario padrao para valores contextuais,
        @return: {dict} dicionario vazio se a atualizacao nao foram ser realizada.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_pcp_ficha_tecnica_capa = self.pool.get('pcp.ficha.tecnica.capa')
        obj_pcp_ficha_tecnica_itens = self.pool.get('pcp.ficha.tecnica.itens')
        obj_wizard_simulacao_precos_insumos = self.pool.get('pcp.wizard.simulacao.precos.insumos')
        obj_adm_utils = self.pool.get('adm.utils')
        obj_model_data = self.pool.get('ir.model.data')
        obj_pcp_util = self.pool.get('pcp.util')

        brw_simulacao_preco = self.browse(cr, uid, ids)[0]
        ficha_id = brw_simulacao_preco.capa_id.id
        produto_id = brw_simulacao_preco.produto_id.id
        quantidade = brw_simulacao_preco.quantidade
        embalagem = brw_simulacao_preco.capa_id.pcpftc_produto_peso_embalagem_id
        quantidade_na_embalagem = embalagem.embalagem_id.quantidade_na_embalagem
        quantidade *= quantidade_na_embalagem
        insumos = self._get_custo_total(cr, uid, ficha_id, context=context)
        preco_venda = brw_simulacao_preco.preco_venda
        custo_total = brw_simulacao_preco.custo_total or 0
        preco_venda_simulacao = brw_simulacao_preco.preco_venda_simulacao or 0.0
        if preco_venda_simulacao <= 0.0:
            raise osv.except_osv(_('Atencao'), _('Valor de preco de venda invalido'))

        adm_parametros = obj_adm_utils.admutl_get_parametros(
            cr, uid, 'adm', ['admpar_siggestao_servidor', 'admpar_siggestao_diretorio',
                             'admpar_siggestao_pv_exportar'], context=context)
        exportar = adm_parametros.get('admpar_siggestao_pv_exportar')
        estoque_parametro = obj_adm_utils.admutl_get_parametros(
            cr, uid, 'estoque', ['estpar_atualizar_custo_fechamentoOP'], context=context)
        atualizar_custo_ordem_producao = estoque_parametro.get(
            'estpar_atualizar_custo_fechamentoOP')
        context['PV'] = True
        context['PC'] = True
        if atualizar_custo_ordem_producao:
            # atualiza os custos no fechamento da ordem de producao
            context['PC'] = False

        if exportar:
            servidor = adm_parametros.get('admpar_siggestao_servidor')
            diretorio = adm_parametros.get('admpar_siggestao_diretorio')
            obj_pcp_util.pcputl_atualizar_pvc_siggestao(cr, uid, produto_id, servidor, diretorio,
                                                        exportar, preco_venda_simulacao,
                                                        custo_total, context)
            reference = obj_model_data.get_object_reference(
                cr, uid, 'pcp', 'wizard_pcp_exporta_preco_venda_form_log')
            view_id = reference and reference[1] or False
            result = {'name': 'Exportacao do preco de venda', 'view_type': 'form',
                      'view_mode': 'form', 'res_model': 'pcp.wizard.simulacao.precos',
                      'view_id': [view_id], 'target': 'new', 'type': 'ir.actions.act_window'}
        else:
            custo_para_erp = insumos / quantidade
            obj_pcp_util.pcputl_atualizar_pvc_sigerp(cr, uid, produto_id, custo_para_erp,
                                                     preco_venda_simulacao, context)
            reference = obj_model_data.get_object_reference(
                cr, uid, 'pcp', 'wizard_pcp_atualiza_preco_venda_form_log')
            view_id = reference and reference[1] or False
            result = {'name': 'Atualizacao do preco de venda', 'view_type': 'form',
                      'view_mode': 'form', 'res_model': 'pcp.wizard.simulacao.precos',
                      'view_id': [view_id], 'target': 'new', 'type': 'ir.actions.act_window'}

        return result

    _columns = {
        'id': fields.integer('Id'),
        'capa_id': fields.many2one("pcp.ficha.tecnica.capa", 'Ficha', required=True),
        'produto_id': fields.many2one("estoque.produto", 'Produto', readonly=True),
        'grade_id': fields.many2one("estoque.produto.grade", 'Grade', readonly=True),
        'produto_embalagem_id': fields.many2one('estoque.produto.embalagem', 'Embalagem',
                                                readonly=True),
        'quantidade': fields.float('Quantidade', digits=(15, 3), readonly=True),
        'preco_venda': fields.float('Preco de venda', digits=(15, 2), readonly=True),
        'preco_custo': fields.float('Preco de custo', digits=(15, 2), readonly=True),
        'custo_total': fields.float('Custo total', digits=(15, 2), readonly=True),
        'encargo_valor': fields.float('Encargos', digits=(15, 2), readonly=True),
        'encargo_porcentagem': fields.float('Encargos %', digits=(15, 2), readonly=True),
        'encargo_total': fields.float('Total dos encargos', digits=(15,2), readonly=True),
        'custo_fixo_porcentagem': fields.float('Custo fixo %', digits=(15, 2), readonly=True),
        'custo_fixo_valor': fields.float('Custo fixo $', digits=(15, 2), readonly=True),
        'comissao': fields.float('Comissao %', digits=(15, 2), readonly=True),
        'comissao_valor': fields.float('Comissao $', digits=(15, 2), readonly=True),
        'lucro_porcentagem': fields.float('% Lucro', digits=(15, 2), readonly=True),
        'preco_venda_simulacao': fields.float('Preco de venda para simulacao', digits=(15, 2)),
        'lucro_simulacao': fields.float('% lucro para simulacao', digits=(15, 2)),
        'margem_minima': fields.float('Margem minima', digits=(15, 2), readonly=True),
        'margem_ideal': fields.float('Margem ideal', digits=(15, 2), readonly=True),
        'margem_bruta': fields.float('Margem bruta', digits=(15, 2), readonly=True),
        'insumo_ids': fields.one2many('pcp.wizard.simulacao.precos.insumos',
                                      'simulacao_capa_id', 'Materia prima &amp; insumos',
                                      readonly=True),
        'encargo_ids': fields.one2many('pcp.wizard.simulacao.precos.encargos', 'encargo_capa_id',
                                       'Encargos', readonly=True)
    }
