#!/usr/bin/python
# -*- encoding: utf-8 -*-

from resposta import Resp
from argos import SigArgosSocket
import binascii
from excecoes import SendCommandTimeout


class SigCommand:
    BYTE_INIT = "02"
    BYTE_END = "03"
    BYTE_START_MESSAGE = "00"

    # response = Resp

    def payload(self):  # pylint: disable=no-self-use
        return ""

    def bytes(self):
        message = self.message()
        return bytearray.fromhex(message)

    def __repr__(self):
        return self.__class__.__name__

    def message(self):
        """
        A mensagem vai no formato:
        INIT_BYTE + MESSAGE_SIZE + BYTE_START_MESSAGE + MESSAGE + CHECKSUM + END_BYTE

        INIT_BYTE = Identifica o início da transmissão, default 0x02
        MESSAGE_SIZE = Quantidade de bytes da payload, dois bytes. o menos significativo fica
        à esquerda, contrariando o padrão. Por ex uma mensagem com payload 421 bytes ficaria
        com o size A5 01.
        CHECKSUM = Representa a integridade da mensagem (ver método self.checksum())
        END_BYTE = Indifica o fim da transmissão, default 0x00

        Todos os valores são representados em uma string hexadecimal, sem separadores.
        A string é convertida para um bytearray ao final do processo, pelo socket.
        """
        payload = self.payload()  # método implementado por cada command
        payload_size = self.payload_size(
            payload
        )  # hexa com a garantia do "0" na frente
        if not isinstance(payload, bytes):
            payload = payload.encode()
        payload_hex = binascii.hexlify(payload)  # pylint: disable=no-member
        start_hex = "{BYTE_INIT}{payload_size}".format(
            BYTE_INIT=SigCommand.BYTE_INIT,
            payload_size=payload_size
        )
        checksum_hex = self.checksum(start_hex, payload_hex)
        end_hex = "{checksum_hex}{BYTE_END}".format(
            checksum_hex=checksum_hex,
            BYTE_END=SigCommand.BYTE_END
        )
        return "{start_hex}{payload_hex}{end_hex}".format(
            start_hex=start_hex,
            payload_hex=payload_hex,
            end_hex=end_hex
        )

    @staticmethod
    def checksum(start_hex, text_hex):
        byte_array = bytearray.fromhex(start_hex + text_hex)
        checksum_byte = byte_array[1]
        for byte in byte_array[2:]:
            checksum_byte = checksum_byte ^ byte
        checksum_byte = checksum_byte & 0xFF
        return "{checksum_byte:02X}".format(checksum_byte=checksum_byte)

    @staticmethod
    def payload_size(payload):
        hex_size = "{res:02X}".format(res=len(payload)).zfill(4)
        str_chunk_size = int(len(hex_size) / 2)
        firstbyte, secondbyte = (
            hex_size[:str_chunk_size],
            hex_size[str_chunk_size:],
        )  # the protocol says the most significant byte should come after (??)
        return secondbyte + firstbyte


class GetEvento():
    def __init__(self, sdk):
        self.sdk = sdk
        self.retorno = ''
        self.data = ''
        self.data_payload = {}
        self.index = False
        """
        Status:
            0 - Executado,
            1 - A executar,
            2 - Timeout,
            3 - Erro
        """
        self.status = 1
        try:
            self.retorno = self.sdk._socket.receive()
        except (
            SendCommandTimeout
        ) as e:
            self.status = 2
            return
        except Exception as e:
            print('Erro de except:', e)
            self.status = 3
            return
        r = Resp(self.retorno)
        self.data = r.data
        self.data_payload = r.data_payload
        self.index = self.data.get('index')
        self.comando = self.data.get('comando')
        self.status = 0


class ComandaLiberarNegarResponse(Resp):
    response_mapping = {"payload": (14, -2, True), "type": (12, 13, False)}


class ComandaLiberarNegar(SigCommand):
    response = ComandaLiberarNegarResponse

    def __init__(self, index, liberaNega='L', mensagens=[], tempoLiberacao=10):
        self.liberaNega = liberaNega if liberaNega in ('L', 'N') else liberaNega == 'L'
        self.index = index
        self.mensagens = mensagens
        self.tempoLiberacao = tempoLiberacao
        self.liberado = 6
        self.negado = 30

    def payload(self):
        comando = self.liberado if self.liberaNega == 'L' else self.negado
        mensagens = ''
        i = 0
        for men in self.mensagens:
            if i == 1:
                mensagens += '}'
            if i == 2:
                mensagens += '{'
            if i == 3:
                mensagens += '}'
            if i > 3:
                break
            mensagens += men
            i += 1
        return "{index}+REON+000+{comando}]{tempoLiberacao}]{mensagens}]".format(
            index=self.index,
            comando=comando,
            tempoLiberacao=self.tempoLiberacao,
            mensagens=mensagens
        )

    def parse_response(self, raw_response):
        return self.response(raw_response)

class sigSetHorario(SigCommand):

    def __init__(self, timestamp):
        self.timestamp = timestamp

    def payload(self):
        timestamp_string = self.timestamp.strftime(SigArgosSocket.TIMESTAMP_MASK)
        message = "01+EH+00+{timestamp_string}]00/00/00]00/00/00".format(timestamp_string=timestamp_string)
        return message

class sigGetHorario(SigCommand):

    def payload(self):
        message = "01+RH+00"
        return message
