#!/usr/bin/python
# -*- coding: utf-8 -*-

from openerp.tests.common import TransactionCase
from sigaddons.config_teste import set_config
import random

__author__ = 'pierre.barbosa'
__version__ = '1.0'
__email_author__ = 'pierre.barbosa[at]siginformatica.com.br'


# noinspection PyUnresolvedReferences
class TestVenlreLiberarRestricoes(TransactionCase):
    @classmethod
    def setUpClass(cls):
        set_config()

    def setUp(self):
        super(TestVenlreLiberarRestricoes, self).setUp()
        # Todas as condicoes de restricao
        self._pessoa_id = self._criar_pessoa_fisica()  # O cliente
        self._criar_cliente_pf(self._pessoa_id)  # Sem limite de credito
        self._restricao_id = self._criar_restricao(self._pessoa_id)

    def test_venlre_restricao_functions_orcamento_bloqueio_mensagem(self):
        """
        Teste de restricao por mensagem em um orcamento.

        :return: n/a
        :rtype: n/a

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        result = self.registry('venlre.liberar.restricoes').venlre_restricao_functions(
            self.cr, self.uid, self._restricao_id, None, list(), context=dict())
        self.assertTrue(result.get(self._restricao_id).get('venlre_bloqueado_por_msg'))

    def test_venlre_restircao_funcions_orcamento_bloqueio_limite_credito(self):
        """
        Teste de restricao por limite de credito em um orcamento.

        :return: n/a
        :rtype: n/a

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        result = self.registry('venlre.liberar.restricoes').venlre_restricao_functions(
            self.cr, self.uid, self._restricao_id, None, list(), context=dict())
        self.assertTrue(result.get(self._restricao_id).get('venlre_bloqueado_limite_credito'))

    def test_venlre_restricao_functions_orcamento_bloqueio_titulo_atraso(self):
        """
        Teste de restricao por titulo em atraso em um orcamento.

        :return: n/a
        :rtype: n/a

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        result = self.registry('venlre.liberar.restricoes').venlre_restricao_functions(
            self.cr, self.uid, self._restricao_id, None, list(), context=dict())
        self.assertTrue(result.get(self._restricao_id).get('venlre_bloqueado_titulo_atrasado'))

    def _criar_restricao(self, pessoa_id):
        """
        Cria restricoes por mensagem, por limite de credito e por titulos atrasados.

        :param pessoa_id: o ID da pessoa
        :type pessoa_id: long
        :return: o ID da restricao que foi criada
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        pessoa_id = pessoa_id or self._pessoa_id
        cliente_id = self.registry('financeiro.cliente').search(
            self.cr, self.uid, [('pessoa_id', '=', pessoa_id)])[0]

        # Restricao por mensagem
        self.registry('financeiro.clientes.mensagem').create(
            self.cr, self.uid, dict(cliente_id=cliente_id, name='MENSAGEM DE VENDA BLOQUEADA',
                                    mensagens_id=self._criar_mensagem_restricao()))

        # Restricao por limite de credito e titulos vencidos
        self._criar_titulos_receber_aberto_vencidos(self._pessoa_id, 6)  # Cria 6 titulos vencidos

        return self.registry('venlre.liberar.restricoes').create(
            self.cr, self.uid, dict(venlre_orcamento_id=self._criar_orcamento(pessoa_id)))

    def _criar_titulos_receber_aberto_vencidos(self, pessoa_id, repeats=1):
        """
        Cria titulos a receber abertos e vencidos (30 dias) para uma pessoa.

        :param pessoa_id: o ID da pessoa
        :type pessoa_id: long
        :param repeats: quantidade de titulos a serem criados
        :return: lista com o ID dos titulos que foram criados
        :rtype: list

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        from dateutil.relativedelta import relativedelta
        from datetime import date
        import time
        pessoa_id = pessoa_id or self._pessoa_id
        titulo_ids = list()
        for repeat in range(1, repeats + 1):
            valor = random.uniform(20., 350.)
            data = date.today()-relativedelta(days=30)
            titulo_ids.append(self.registry('financeiro.titulo').create(
                self.cr, self.uid, dict(pessoa_id=pessoa_id, saldo=valor, vr_original=valor,
                                        data_prorrogacao=data, data_vencimento=data,
                                        data_emissao=data, state='A', tipo_titulo='R',
                                        tipo_lancamento='A',
                                        documento=time.strftime('%Y%m%d%H%M%S'),
                                        doc_tipo_id=self._criar_tipo_documento_orcamento())))

        return titulo_ids[0] if repeats == 1 else titulo_ids

    def _criar_tipo_documento_orcamento(self):
        """
        Cria um tipo de documento "orcamento" (ORC).

        :return: o ID do documento que foi criado
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        result = self.registry('documento.documento.tipo').search(
            self.cr, self.uid, [('sigla', '=', 'ORC')])
        if not result:
            result = self.registry('documento.documento.tipo').create(
                self.cr, self.uid, dict(bloqueio=True, descricao='Orcamento', sigla='ORC',
                                        liquidavel=True))
        else:
            result = result[0]

        return result

    def _criar_mensagem_restricao(self):
        """
        Cria mensagem de restricao e atribui a uma pessoa.

        :return: o ID da mensagem de restricao
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        result = self.registry('financeiro.clientes.mensagem.nome').search(
            self.cr, self.uid, [('codigo', '=', 12345)], context=dict())
        if not result:
            result = self.registry('financeiro.clientes.mensagem.nome').create(
                self.cr, self.uid, dict(codigo=12345, name='VENDA BLOQUEADA', tipo_mensagem='B'),
                dict())
        else:
            result = result[0]

        return result

    def _criar_orcamento(self, pessoa_id):
        """
        Cria um orcamento.

        :param pessoa_id: o ID da pessoa
        :type pessoa_id: long
        :return: o ID do orcamento que foi criado
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        pessoa_id = pessoa_id or self._pessoa_id

        # Tres itens serao criados
        values = dict(pessoa_id=pessoa_id, orcamento_itens_ids=[
            [0, 0, dict(local_id=self._criar_local(), produto_id=_id,
                        produto_embalagem_id=self._criar_produto_embalagem(_id),
                        orcori_preco_venda=random.uniform(1.0, 99.9),
                        quantidade=random.randint(1, 10), valor_total=random.uniform(1.0, 99.9))]
            for _id in self._criar_item_produto(3)])

        # Atribui uma locacao para grade
        for item in values['orcamento_itens_ids']:
            grade_id = self.registry('estoque.produto.grade').search(
                self.cr, self.uid, [('produto_id', '=', item[2]['produto_id'])], context=dict())[0]
            item[2]['grade_id'] = grade_id  # Ajusta a grade
            self.registry('estoque.local.locacao').create(
                self.cr, self.uid, dict(
                    produto_id=item[2]['produto_id'], grade_id=item[2]['grade_id'],
                    local_id=item[2]['local_id']))

        return self.registry('orcamentos.orcamento.capa').create(
            self.cr, self.uid, values, context=dict())

    def _criar_cliente_pf(self, pessoa_id):
        """
        Cria um cliente pessoa fisica sem limite de credito.

        :param pessoa_id: o Id da pessoa
        :type pessoa_id: long
        :return: o ID do cliente
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        pessoa_id = pessoa_id or self._pessoa_id
        result = self.registry('financeiro.cliente').search(
            self.cr, self.uid, [('pessoa_id', '=', pessoa_id)], context=dict())
        if not result:
            result = self.registry('financeiro.cliente').create(
                self.cr, self.uid, dict(pessoa_id=pessoa_id, limite_credito=0))
        else:
            result = result[0]

        return result

    def _criar_pessoa_fisica(self):
        """
        Cria uma pessoa fisica. Utiliza um CPF valido.

        :return: o ID da pessoa que foi criado
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        # Se o CPF do cliente a ser criado jah existe, nao cria novo registro
        result = self.registry('adm.pessoa').search(
            self.cr, self.uid, [('cpf_cnpj', '=', '37371541120')], context=dict())
        if not result:
            result = self.registry('adm.pessoa').create(
                self.cr, self.uid, dict(pessoa_tipo='F', name='Pierre Geraldo Moura Barbosa',
                                        cpf_cnpj='37371541120', admpes_fis_sexo='M'))
        else:
            result = result[0]

        return result

    def _criar_item_produto(self, repeats=1):
        """
        Cria um ou mais produtos.

        :param repeats: o numero de produtos a ser criado
        :return: o ID do produto que foi criado
        :rtype:long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        produto_ids = list()
        for repeat in range(1, repeats + 1):
            descricao = 'Produto %d de uso interno (nao especificado)' % repeat
            result = self.registry('estoque.produto').search(
                self.cr, self.uid, [('name', '=', descricao)])
            if not result:  # Cria o produto se nao encontrar um com a mesma descricao
                produto_ids.append(self.registry('estoque.produto').create(
                    self.cr, self.uid, dict(name=descricao, ncm_id=self._criar_ncm(), tipo='P',
                                            preco_venda_ids=[[0, 0, dict(
                                                preco_venda=random.uniform(1.0, 50.9))]])))
            else:
                produto_ids.append(result[0])

        return produto_ids[0] if repeats == 1 else produto_ids

    def _criar_ncm(self):
        """
        Cria NCM para um item (produto/servico).

        :return: o ID do NCM que foi criado.
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        # Se o NCM do item existe, nao cria um novo
        result = self.registry('fiscal.ncm').search(
            self.cr, self.uid, [('codigo', '=', '01010199')], context=dict())
        if not result:
            result = self.registry('fiscal.ncm').create(
                self.cr, self.uid, dict(codigo='01010199', descricao='01010199'))
        else:
            result = result[0]

        return result

    def _criar_local(self):
        """
        Cria um local para o item (produto/servico).

        :return: o ID do local que foi criado.
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        # Se o local existe, nao cria um novo
        result = self.registry('estoque.local').search(
            self.cr, self.uid, [('numero', '=', 1)], context=dict())
        if not result:
            centro_custo_id = self._criar_centro_custo()
            result = self.registry('estoque.local').create(
                self.cr, self.uid, dict(name='LOJA', numero=1, centrocusto_id=centro_custo_id),
                dict())
        else:
            result = result[0]

        return result

    def _criar_centro_custo(self):
        """
        Cria um centro de custo para o local.

        :return: o ID do centro de custo que foi criado.
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        # Se o centro de custo existe, nao cria um novo
        result = self.registry('estoque.local').search(
            self.cr, self.uid, [('codigo', '=', '0')], context=dict())
        if not result:
            result = self.registry('estoque.local').create(
                self.cr, self.uid, dict(name='Administracao', sigla='ADM'))
        else:
            result = result[0]

        return result

    def _criar_produto_embalagem(self, produto):
        """
        Cria uma embalagem e atribui ao produto.

        :param produto: o ID do produto
        :return: o ID da embalagem do produto.
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        embalagem_id = self._criar_embalagem()
        result = self.registry('estoque.produto.embalagem').search(
            self.cr, self.uid, [('produto_id', '=', produto), ('embalagem_id', '=', embalagem_id)],
            context=dict())
        if not result:
            result = self.registry('estoque.produto.embalagem').create(
                self.cr, self.uid, dict(produto_id=produto, embalagem_id=embalagem_id))
        else:
            result = result[0]

        return result

    def _criar_embalagem(self):
        """
        Cria uma embalagem.

        :return: o ID da embalagem.
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        # Se a embalagem existe, nao cria uma nova
        result = self.registry('estoque.embalagem').search(
            self.cr, self.uid, [('name', '=', 'UNIDADE')], context=dict())
        if not result:
            unidade_id = self._criar_unidade()
            result = self.registry('estoque.embalagem').create(
                self.cr, self.uid, dict(name='UNIDADE', sigla='UN', unidade_id=unidade_id,
                                        quantidade_na_embalagem=1))
        else:
            result = result[0]

        return result

    def _criar_unidade(self):
        """
        Cria uma unidade para a embalagem.

        :return: o ID da unidade.
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        # Se a embalagem existe, nao cria uma nova
        result = self.registry('estoque.embalagem.unidade').search(
            self.cr, self.uid, [('name', '=', 'UNIDADE')], context=dict())
        if not result:
            result = self.registry('estoque.embalagem.unidade').create(
                self.cr, self.uid, dict(name='UNIDADE', unidade='UN'))
        else:
            result = result[0]

        return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
